@@ -1,6 +1,8 @@
 require "log4r"
 require "vagrant/util/retryable"
+require "vagrant/util/platform"
 require "tempfile"
+require Vagrant.source_root.join("plugins/synced_folders/unix_mount_helpers")
 
 # This is already done for us in lib/vagrant-sshfs.rb. We needed to
 # do it there before Process.uid is called the first time by Vagrant
@@ -14,15 +16,20 @@ module VagrantPlugins
     module Cap
       class MountSSHFS
         extend Vagrant::Util::Retryable
+        extend VagrantPlugins::SyncedFolder::UnixMountHelpers
         @@logger = Log4r::Logger.new("vagrant::synced_folders::sshfs_mount")
 
+        def self.list_mounts_command
+          "cat /proc/mounts"
+        end
+
         def self.sshfs_forward_is_folder_mounted(machine, opts)
           mounted = false
           guest_path = opts[:guestpath]
 
           # If the path doesn't exist at all in the machine then we
           # can safely say it is not mounted
-          exists = machine.communicate.test("test -e #{guest_path}")
+          exists = machine.communicate.test("test -e #{guest_path}", sudo: true)
           return false unless exists
 
           # find the absolute path so that we can properly check if it is mounted
@@ -31,7 +38,7 @@ module VagrantPlugins
             :sshfs_get_absolute_path, guest_path)
 
           # consult /proc/mounts to see if it is mounted or not
-          machine.communicate.execute("cat /proc/mounts") do |type, data|
+          machine.communicate.execute(self.list_mounts_command) do |type, data|
             if type == :stdout
               data.each_line do |line|
                 if line.split()[1] == absolute_guest_path
@@ -48,7 +55,7 @@ module VagrantPlugins
           # opts contains something like:
           #   { :type=>:sshfs,
           #     :guestpath=>"/sharedfolder",
-          #     :hostpath=>"/guests/sharedfolder", 
+          #     :hostpath=>"/guests/sharedfolder",
           #     :disabled=>false
           #     :ssh_host=>"192.168.1.1"
           #     :ssh_port=>"22"
@@ -66,16 +73,41 @@ module VagrantPlugins
             comm.sudo("chmod 777 #{expanded_guest_path}")
           end
 
-          # Mount path information
-          hostpath = opts[:hostpath].dup
-          hostpath.gsub!("'", "'\\\\''")
+          # Mount path information: if arbitrary host mounting then
+          # take as is. If not, then expand the hostpath to the real
+          # path.
+          if opts[:ssh_host]
+              hostpath = opts[:hostpath].dup
+          else
+              hostpath = File.expand_path(opts[:hostpath], machine.env.root_path)
+              hostpath  = Vagrant::Util::Platform.fs_real_path(hostpath).to_s
+          end
+
+          # Support for user provided mount_options, owner, group
+          # https://github.com/hashicorp/vagrant/blob/2c3397c46851ef29a3589bf3214a3eee12da8484/website/content/docs/synced-folders/basic_usage.mdx#options
+          mount_options = opts.fetch(:mount_options, [])
+          # Determine owner/group info to use
+          if (opts.has_key?(:owner) and opts[:owner]) or
+             (opts.has_key?(:group) and opts[:group])
+            detected_ids = detect_owner_group_ids(
+                machine, expanded_guest_path, mount_options, opts)
+            mount_uid = detected_ids[:uid]
+            mount_gid = detected_ids[:gid]
+            mount_options.append("uid=#{mount_uid}")
+            mount_options.append("gid=#{mount_gid}")
+          end
+          # Combine mount_options into sshfs_opts_append (also user provided)
+          if not mount_options.empty?()
+            opts[:sshfs_opts_append] =
+              opts[:sshfs_opts_append].to_s + ' -o ' + mount_options.join(',') + ' '
+          end
 
           # Add in some sshfs/fuse options that are common to both mount methods
           opts[:sshfs_opts] = ' -o allow_other ' # allow non-root users to access
           opts[:sshfs_opts]+= ' -o noauto_cache '# disable caching based on mtime
 
           # Add in some ssh options that are common to both mount methods
-          opts[:ssh_opts] = ' -o StrictHostKeyChecking=no '# prevent yes/no question 
+          opts[:ssh_opts] = ' -o StrictHostKeyChecking=no '# prevent yes/no question
           opts[:ssh_opts]+= ' -o ServerAliveInterval=30 '  # send keepalives
 
           # Do a normal mount only if the user provided host information
@@ -116,25 +148,25 @@ module VagrantPlugins
         protected
 
         def self.windows_uninherit_handles(machine)
-          # For win32-process Process.create, if we pass any file handles to the 
-          # underlying process for stdin/stdout/stderr then all file handles are 
+          # For win32-process Process.create, if we pass any file handles to the
+          # underlying process for stdin/stdout/stderr then all file handles are
           # inherited by default. We'll explicitly go through and set all Handles
           # to not be inheritable by default. See following links for more info
-          # 
+          #
           # https://github.com/djberg96/win32-process/blob/6b380f450aebb69d44bb7accd958ecb6b9e1d246/lib/win32/process.rb#L445-L447
           # bInheritHandles from https://msdn.microsoft.com/en-us/library/windows/desktop/ms682425(v=vs.85).aspx
-          # 
+          #
           # In 6f285cd we made it so that we would uninherit all filehandles by
           # default on windows. Some users have reported that this operation
           # is erroring because `The parameter is incorrect.`. See #52
-          # We will make the uninheriting operation best effort. The rationale 
-          # is that if a file handle was not able to be set to uninheritable 
+          # We will make the uninheriting operation best effort. The rationale
+          # is that if a file handle was not able to be set to uninheritable
           # then it probably wasn't one that would get inherited in the first place.
           #
-          # For each open IO object 
+          # For each open IO object
           ObjectSpace.each_object(IO) do |io|
             if !io.closed?
-              fileno = io.fileno 
+              fileno = io.fileno
               @@logger.debug("Setting file handle #{fileno} to not be inherited")
               begin
                 self.windows_uninherit_handle(fileno)
@@ -151,7 +183,7 @@ module VagrantPlugins
 
         def self.windows_uninherit_handle(fileno)
           # Right now we'll be doing this using private methods from the win32-process
-          # module by calling  For each open IO object. Much of this code was copied from 
+          # module by calling  For each open IO object. Much of this code was copied from
           # that module. We access the private methods by using the object.send(:method, args)
           # technique. In the future we want to get a patch upstream so we don't need to
           # access private methods. Upstream request is here:
@@ -170,15 +202,15 @@ module VagrantPlugins
           end
 
           # Now clear the HANDLE_FLAG_INHERIT from the HANDLE so that the handle
-          # won't get shared by default. See: 
+          # won't get shared by default. See:
           # https://msdn.microsoft.com/en-us/library/windows/desktop/ms724935(v=vs.85).aspx
-          # 
+          #
           bool = Process.send(:SetHandleInformation, handle,
                               Process::Constants::HANDLE_FLAG_INHERIT, 0)
           raise SystemCallError.new("SetHandleInformation", FFI.errno) unless bool
         end
 
-        # Perform a mount by running an sftp-server on the vagrant host 
+        # Perform a mount by running an sftp-server on the vagrant host
         # and piping stdin/stdout to sshfs running inside the guest
         def self.sshfs_slave_mount(machine, opts, hostpath, expanded_guest_path)
 
@@ -198,24 +230,35 @@ module VagrantPlugins
 
           # The remote sshfs command that will run (in slave mode)
           sshfs_opts+= ' -o slave '
-          sshfs_cmd = "sudo -E sshfs :#{hostpath} #{expanded_guest_path}" 
+          sshfs_cmd = "sudo -E sshfs :#{hostpath} #{expanded_guest_path}"
           sshfs_cmd+= sshfs_opts + ' ' + sshfs_opts_append + ' '
 
           # The ssh command to connect to guest and then launch sshfs
-          # Note the backslash escapes for IdentityFile - handles spaces in key path
           ssh_opts = opts[:ssh_opts]
           ssh_opts+= ' -o User=' + machine.ssh_info[:username]
           ssh_opts+= ' -o Port=' + machine.ssh_info[:port].to_s
-          ssh_opts+= ' -o "IdentityFile=\"' + machine.ssh_info[:private_key_path][0] + '\""'
           ssh_opts+= ' -o UserKnownHostsFile=/dev/null '
           ssh_opts+= ' -F /dev/null ' # Don't pick up options from user's config
+          if machine.ssh_info.key?(:private_key_path) and
+             machine.ssh_info[:private_key_path] and
+             machine.ssh_info[:private_key_path][0]
+             # Add IdentityFile since there is one
+             # Note the backslash escapes for IdentityFile - handles spaces in key path
+            ssh_opts+= ' -o "IdentityFile=\"' + machine.ssh_info[:private_key_path][0] + '\""'
+          end
+
+          # Use an SSH ProxyCommand when corresponding Vagrant setting is defined
+          if machine.ssh_info[:proxy_command]
+            ssh_opts+= " -o ProxyCommand=\"" + machine.ssh_info[:proxy_command] + "\""
+          end
+
           ssh_cmd = ssh_path + ssh_opts + ' ' + ssh_opts_append + ' ' + machine.ssh_info[:host]
           ssh_cmd+= ' "' + sshfs_cmd + '"'
 
           # Log some information
           @@logger.debug("sftp-server cmd: #{sftp_server_cmd}")
           @@logger.debug("ssh cmd: #{ssh_cmd}")
-          machine.ui.info(I18n.t("vagrant.sshfs.actions.slave_mounting_folder", 
+          machine.ui.info(I18n.t("vagrant.sshfs.actions.slave_mounting_folder",
                           hostpath: hostpath, guestpath: expanded_guest_path))
 
           # Create two named pipes for communication between sftp-server and
@@ -234,8 +277,8 @@ module VagrantPlugins
           # The way this works is by hooking up the stdin+stdout of the
           # sftp-server process to the stdin+stdout of the sshfs process
           # running inside the guest in slave mode. An illustration is below:
-          # 
-          #          stdout => w1      pipe1         r1 => stdin 
+          #
+          #          stdout => w1      pipe1         r1 => stdin
           #         />------------->==============>----------->\
           #        /                                            \
           #        |                                            |
@@ -243,7 +286,7 @@ module VagrantPlugins
           #        |                                            |
           #        \                                            /
           #         \<-------------<==============<-----------</
-          #          stdin <= r2        pipe2         w2 <= stdout 
+          #          stdin <= r2        pipe2         w2 <= stdout
           #
           # Wire up things appropriately and start up the processes
           if Vagrant::Util::Platform.windows?
@@ -265,8 +308,8 @@ module VagrantPlugins
                            :thread_inherit => true,
                            :startup_info => {:stdin => w1, :stdout => r2, :stderr => f2})
           else
-            p1 = spawn(sftp_server_cmd, :out => w2, :in => r1, :err => f1, :pgroup => true)
-            p2 = spawn(ssh_cmd,         :out => w1, :in => r2, :err => f2, :pgroup => true)
+            p1 = spawn(sftp_server_cmd, :out => w2, :in => r1, :err => f1, :pgroup => true, :close_others => true)
+            p2 = spawn(ssh_cmd,         :out => w1, :in => r2, :err => f2, :pgroup => true, :close_others => true)
 
             # Detach from the processes so they will keep running
             Process.detach(p1)
@@ -319,13 +362,13 @@ module VagrantPlugins
           end
 
           # Log some information
-          machine.ui.info(I18n.t("vagrant.sshfs.actions.normal_mounting_folder", 
-                          user: username, host: host, 
+          machine.ui.info(I18n.t("vagrant.sshfs.actions.normal_mounting_folder",
+                          user: username, host: host,
                           hostpath: hostpath, guestpath: expanded_guest_path))
-          
+
           # Build up the command and connect
           error_class = VagrantPlugins::SyncedFolderSSHFS::Errors::SSHFSNormalMountFailed
-          cmd = echopipe 
+          cmd = echopipe
           cmd+= "sshfs -p #{port} "
           cmd+= ssh_opts + ' ' + ssh_opts_append + ' '
           cmd+= sshfs_opts + ' ' + sshfs_opts_append + ' '
