@@ -1,3 +1,4 @@
+require "etc"
 require "log4r"
 require "vagrant/util/retryable"
 require "tempfile"
@@ -29,7 +30,7 @@ module VagrantPlugins
           # opts contains something like:
           #   { :type=>:sshfs,
           #     :guestpath=>"/sharedfolder",
-          #     :hostpath=>"/guests/sharedfolder", 
+          #     :hostpath=>"/guests/sharedfolder",
           #     :disabled=>false
           #     :ssh_host=>"192.168.1.1"
           #     :ssh_port=>"22"
@@ -45,7 +46,7 @@ module VagrantPlugins
 
         protected
 
-        # Perform a mount by running an sftp-server on the vagrant host 
+        # Perform a mount by running an sftp-server on the vagrant host
         # and piping stdin/stdout to sshfs running inside the guest
         def self.sshfs_mount(machine, opts)
 
@@ -55,6 +56,12 @@ module VagrantPlugins
           expanded_guest_path = machine.guest.capability(
             :shell_expand_guest_path, opts[:guestpath])
 
+          # Create the mountpoint inside the guest
+          machine.communicate.tap do |comm|
+            comm.sudo("mkdir -p #{expanded_guest_path}")
+            comm.sudo("chmod 777 #{expanded_guest_path}")
+          end
+
           # Mount path information
           hostpath = opts[:hostpath].dup
           hostpath.gsub!("'", "'\\\\''")
@@ -63,22 +70,53 @@ module VagrantPlugins
           opts[:sshfs_opts] = ' -o noauto_cache '# disable caching based on mtime
 
           # Add in some ssh options that are common to both mount methods
-          opts[:ssh_opts] = ' -o StrictHostKeyChecking=no '# prevent yes/no question 
+          opts[:ssh_opts] = ' -o StrictHostKeyChecking=no '# prevent yes/no question
           opts[:ssh_opts]+= ' -o ServerAliveInterval=30 '  # send keepalives
 
           # SSH connection options
-          # Note the backslash escapes for IdentityFile - handles spaces in key path
           ssh_opts = opts[:ssh_opts]
           ssh_opts+= ' -o Port=' + machine.ssh_info[:port].to_s
-          ssh_opts+= ' -o "IdentityFile=\"' + machine.ssh_info[:private_key_path][0] + '\""'
           ssh_opts+= ' -o UserKnownHostsFile=/dev/null '
           ssh_opts+= ' -F /dev/null ' # Don't pick up options from user's config
+          if machine.ssh_info.key?(:private_key_path) and
+             machine.ssh_info[:private_key_path] and
+             machine.ssh_info[:private_key_path][0]
+             # Add IdentityFile since there is one
+             # Note the backslash escapes for IdentityFile - handles spaces in key path
+            ssh_opts+= ' -o "IdentityFile=\"' + machine.ssh_info[:private_key_path][0] + '\""'
+          end
 
           ssh_opts_append = opts[:ssh_opts_append].to_s # provided by user
 
+          # Support for user provided mount_options, owner, group
+          # https://github.com/hashicorp/vagrant/blob/2c3397c46851ef29a3589bf3214a3eee12da8484/website/content/docs/synced-folders/basic_usage.mdx#options
+          mount_options = opts.fetch(:mount_options, [])
+          if (opts.has_key?(:owner) and opts[:owner]) or
+             (opts.has_key?(:group) and opts[:group])
+            # Identify the uid
+            if opts.has_key?(:owner) and opts[:owner]
+              mount_uid = Etc::getpwnam(opts[:owner]).uid
+            else
+              mount_uid = Etc::getpwnam(Etc.getlogin).uid
+            end
+            # Identify the gid. If a group was provided use that otherwise use
+            # the group detected with the detected user id.
+            if opts.has_key?(:group) and opts[:group]
+              mount_gid = Etc::getgrnam(opts[:group]).gid
+            else
+              mount_gid = Etc::getpwnam(Etc.getlogin).gid
+            end
+            # Add them to the mount options
+            mount_options.append("uid=#{mount_uid}")
+            mount_options.append("gid=#{mount_gid}")
+          end
+
           # SSHFS executable options
           sshfs_opts = opts[:sshfs_opts]
           sshfs_opts_append = opts[:sshfs_opts_append].to_s # provided by user
+          if not mount_options.empty?()
+            sshfs_opts_append+= ' -o ' + mount_options.join(',') + ' '
+          end
 
           username = machine.ssh_info[:username]
           host = machine.ssh_info[:host]
@@ -87,12 +125,12 @@ module VagrantPlugins
           # The sshfs command to mount the guest directory on the host
           sshfs_cmd = "#{sshfs_path} #{ssh_opts} #{ssh_opts_append} "
           sshfs_cmd+= "#{sshfs_opts} #{sshfs_opts_append} "
-          sshfs_cmd+= "#{username}@#{host}:#{expanded_guest_path} #{hostpath}" 
+          sshfs_cmd+= "#{username}@#{host}:#{expanded_guest_path} #{hostpath}"
 
           # Log some information
           @@logger.debug("sshfs cmd: #{sshfs_cmd}")
 
-          machine.ui.info(I18n.t("vagrant.sshfs.actions.reverse_mounting_folder", 
+          machine.ui.info(I18n.t("vagrant.sshfs.actions.reverse_mounting_folder",
                           hostpath: hostpath, guestpath: expanded_guest_path))
 
           # Log STDERR to predictable files so that we can inspect them
@@ -103,7 +141,7 @@ module VagrantPlugins
 
           # Launch sshfs command to mount guest dir into the host
           if Vagrant::Util::Platform.windows?
-            # Need to handle Windows differently. Kernel.spawn fails to work, 
+            # Need to handle Windows differently. Kernel.spawn fails to work,
             # if the shell creating the process is closed.
             # See https://github.com/dustymabe/vagrant-sshfs/issues/31
             Process.create(:command_line => ssh_cmd,
@@ -112,7 +150,7 @@ module VagrantPlugins
                            :thread_inherit => true,
                            :startup_info => {:stdin => w2, :stdout => r1, :stderr => f1})
           else
-            p1 = spawn(sshfs_cmd,   :out => f1, :err => f1, :pgroup => true)
+            p1 = spawn(sshfs_cmd,   :out => f1, :err => f1, :pgroup => true, :close_others => true)
             Process.detach(p1) # Detach so process will keep running
           end
 
